#!/bin/sh

ROOT_DIR="./"
KMOD_DIR="./"

oaf_ipk_list="
appfilter
luci-app-oaf
luci-i18n-oaf-zh-cn
"

depend_ipk_list="
libjson-c
libubox
libblobmsg-json
libuci
libubus"

uni_ipk_list="
luci-i18n-oaf-zh-cn
luci-app-oaf
appfilter
kmod-oaf
"
LOG_FILE="/tmp/log/oaf_install.log"

old_kmod_file="/etc/oaf_backup.ko"
kernel_ver=`uname -a | awk '{print $3}'`
log()
{
	echo "$1" >>$LOG_FILE
}

install_ipk()
{
	local ret=0
	local ipk_name=$1
	if [ x"" == x"$ipk_name" ];then
		return 3
	fi
	ipk=`ls  |grep ^$ipk_name`
	if [ x"" == x"$ipk" ];then
		log "$ipk_name not exist"
		return 2
	fi
	opkg install $ipk 2>>$LOG_FILE
	if [ $? -ne 0 ];then
		return 1
	fi
	return 0
}

uninstall()
{
	local sys_kernel_ver=`uname -a |awk '{print $3}' | awk -F- '{print $1}'`
	for ipk in $uni_ipk_list;do
		echo "uninstall ipk $ipk"
		opkg remove $ipk
	done
	if [ -f /lib/modules/oaf.ko ];then
		rm /lib/modules/oaf.ko
	fi
	rmmod oaf
	rm /lib/modules/oaf.ko
	rm /tmp/luci* -fr
}


backup_old_kmod()
{
	if [ -f "/lib/modules/$kernel_ver/oaf.ko" ];then
		cp /lib/modules/$kernel_ver/oaf.ko $old_kmod_file
		log "backup old oaf kmod /lib/modules/$kernel_ver/oaf.ko ok"
	elif [ -f "/lib/modules/oaf.ko" ];then
		cp /lib/modules/oaf.ko $old_kmod_file
		log "backup old oaf kmod /lib/modules/oaf.ko ok"
	fi
}


install()
{
	echo "">>/tmp/feature.cfg
	if [ ! -f oaf.ko ];then
		local kmod_name=`ls oaf*`
		cp $kmod_name oaf.ko
	fi
	lsmod |grep oaf>/dev/null
	if [ $? -ne 0 ];then
		insmod oaf.ko >/dev/null
		if [ $? -ne 0 ];then
			log "install oaf kmod....failed"
			if [ -f $old_kmod_file ];then
				insmod $old_kmod_file >/dev/null
				if [ $? -ne 0 ];then
					log "驱动加载失败，和当前内核版本不兼容，建议使用openwrt官方稳定版本"
					return 1
				else
					log "新版本驱动加载失败，使用旧版本驱动"
					cp $old_kmod_file ./oaf.ko
				fi
			else
				log "驱动加载失败，和当前内核版本不兼容，建议使用openwrt官方稳定版本"
				return 1
			fi
		fi
	fi
	log "install oaf kmod....ok"
	cp oaf.ko /lib/modules/
	
	if [ -d other_ipks ];then
		cd other_ipks
		for ipk in $depend_ipk_list;do
			log "install depend ipk $ipk"
			install_ipk $ipk
		done
		cd -
	fi
	
	
	for ipk in $oaf_ipk_list;do
		install_ipk $ipk
		if [ $? -ne 0 ];then
			log "install $ipk failed"
			return 1
		fi
		log "install $ipk ...ok"
	done

	cd -
	rm /tmp/luci* -fr
	return 0
}



action=$1
case $action in
"install")
	install
	if [ $? -eq 0 ];then
		/etc/init.d/appfilter restart
		exit 0
	else
		exit 1
	fi
;;
"uninstall")
	/etc/init.d/appfilter stop
	backup_old_kmod
	#check oaf load status
	uninstall
	if [ $? -eq 0 ];then
		exit 0
	else
		exit 1
	fi
;;
*)
	echo "param error"
;;
esac
exit 0
