
local http = require "luci.http"
local fs = require "nixio.fs"
local sys = require "luci.sys"
local uci = require"luci.model.uci".cursor()
local util = require "luci.util"

wget = "/usr/bin/wget"
wget_args = {"--no-check-certificate", "--quiet", "--timeout=100", "--tries=3"}
local oaf_api = "http://oaf.ttcoder.cn:8082/api/get_ipk_info"
module("luci.controller.oaf_helper", package.seeall)
function index()
	entry({"admin", "services", "oaf_helper"}, cbi("oaf_helper", {hideapplybtn=true, hidesavebtn=true, hideresetbtn=true}), "防沉迷助手" , 1)
    entry({"admin", "services", "oaf_helper", "uninstall_oaf_ipk"}, call("uninstall_oaf_ipk_handle")).leaf = true
    entry({"admin", "services", "oaf_helper", "get_oaf_status"}, call("get_oaf_status_handle")).leaf = true
    entry({"admin", "services", "oaf_helper", "get_ipk_list"}, call("get_ipk_list_handle")).leaf = true
    entry({"admin", "services", "oaf_helper", "install_ipk"}, call("install_ipk_handle")).leaf = true
end



function f_log(log_text)
    local log_file = "/tmp/log/oaf_helper.log"
    
    local datetime = os.date("%Y-%m-%d %H:%M:%S")
    local log_content = string.format("[%s] %s\n", datetime, log_text)
    local file = io.open(log_file, "a")
    if file then
        file:write(log_content)
        file:close()
    end
end


function get_oaf_version()
    local version = sys.exec("cat /etc/version")
    if version == nil then
        return ""
    else
        return version
    end
end

function check_old_version()
    local resp_json= {
    }
    local oaf_status = sys.exec("opkg list-installed |grep oaf")
    if oaf_status == nil or oaf_status == ""  then
    	resp_json.code = 40000
    else
    	resp_json.code = 20000 
    end
    http.prepare_content("application/json") 
    http.write_json(resp_json)
end

function get_new_version()
    local version = sys.exec("cat /tmp/new_version")
    if version == nil then
        return ""
    else
        return version
    end
end

function get_install_log()
    local log_text = sys.exec("cat /tmp/log/oaf_install.log")
    local resp_json= {
    }
    http.prepare_content("application/json") 
    resp_json.code = 20000
    resp_json.data = log_text
    http.write_json(resp_json)
end

function get_board_info()
    local board=util.trim(luci.sys.exec("cat /etc/os-release  | grep OPENWRT_BOARD | awk -F= '{print $2}' | xargs | awk -F/ '{print $1 \"_\" $2}'") )
    if board == nil then
        return ""
    else
        return board
    end
end

function get_kernel_version()
    return util.trim(luci.sys.exec("uname -a | awk '{print $3}'"))
end


function get_api_json(url)
    local jsonc = require "luci.jsonc"
    luci.sys.exec("rm /tmp/info.json")
    luci.sys.exec(wget.." '"..url.."'".. " -O /tmp/info.json -T 3")
    local json_content = luci.sys.exec("cat /tmp/info.json")
    if json_content == "" then 
        f_log("get api json failed, url: "..url)
        return nil 
    end

    local json_obj = jsonc.parse(json_content)
    if json_obj == nil then
        f_log("parse api json failed, content: "..json_content)
        return nil
    end
    return json_obj
end

function version_check(arch)
    local json = get_api_json(oaf_api)

    resp_json= {
        code = '',
        url = {html = "", download = ""}
    }
    http.prepare_content("application/json")                     
    http.write_json(json)
end

function upgrade_oaf_ipk(arch)
    local resp_json= {
        code=40005,
        data={}
    }
    http.prepare_content("application/json") 

    local board = get_board_info()
    local kernel = get_kernel_version()
    if board == nil or kernel == nil then
        resp_json.code = 40006
        http.write_json(resp_json)
        return
    end
  
    local url = oaf_api.."?kernel="..kernel.."&arch="..board
    local json = get_api_json(url)
    if json == nil then
        http.write_json(resp_json)
        return
    end

    if json.code ~= 20000 then
        -- kernel
        if json.code == 40002 then 
            resp_json.code = 40008
            resp_json.msg = json.msg
        else
            -- arch
            resp_json.code = 40007 
            resp_json.msg = json.msg
        end
        http.write_json(resp_json)
        return
    end
   
    local result=util.trim(luci.sys.exec("oaf_mgr upgrade "..json.data.url));
    code=tonumber(result)
    resp_json.code = code
    http.write_json(resp_json)
end

function uninstall_oaf_ipk_handle()
    local resp_json= {
        code=20000,
    }


    http.prepare_content("application/json") 
 
    luci.sys.exec("oaf.sh uninstall");
    luci.sys.exec("opkg remove luci-i18n-oaf-zh-cn");
    luci.sys.exec("opkg remove luci-app-oaf");
    luci.sys.exec("opkg remove appfilter");
    luci.sys.exec("opkg remove kmod-oaf");
    luci.sys.exec("opkg remove luci-i18n-appfilter-zh-cn ");
    luci.sys.exec("opkg remove luci-app-appfilter");
    http.write_json(resp_json)
end

function get_oaf_status_handle()
    
    local resp_json= {
        code=20000,
        data={}
    }
    http.prepare_content("application/json") 
    local version = sys.exec("opkg list-installed |grep ^appfilter | awk -F- '{print $2}'")
    if version == nil or version == ""  then
        resp_json.data.version = ""
    else
        resp_json.data.version = version
    end
    local driver_version = sys.exec("cat /proc/sys/oaf/version")
    if driver_version == nil or driver_version == ""  then
        resp_json.data.driver_version = "5.0"
    else
        resp_json.data.driver_version = driver_version
    end


    local result = sys.exec("lsmod |grep ^oaf")
    if result == nil or result == ""  then
        resp_json.data.driver_status = 0
    else
        resp_json.data.driver_status = 1
    end


    resp_json.data.board = get_board_info() 
    resp_json.data.kernel = get_kernel_version()

    resp_json.code = 20000 
    http.write_json(resp_json)
end


function get_ipk_list_handle()
    local resp_json= {
        code=20000,
        data={}
    }
    http.prepare_content("application/json") 
    local board = get_board_info()
    local kernel = get_kernel_version()
    if board == nil or kernel == nil then
        resp_json.code = 40001
        resp_json.msg = "get board info or kernel version failed"
        http.write_json(resp_json)
        return
    end
    local fw= luci.http.formvalue("fw")                             
    local url = oaf_api.."?kernel="..kernel.."&arch="..board.."&fw="..fw.."&v=1.0.1"
    f_log("get ipk list url: "..url)

    local json = get_api_json(url)
    if json == nil then
        resp_json.code = 40002
        resp_json.msg = "network error or server is under maintenance"
        http.write_json(resp_json)
        return
    end
    http.write_json(json)
end



function install_ipk_handle()
    f_log("install ipk")
    local result_file = "/tmp/oaf_mgr_result"
    local result_log_file = "/tmp/log/oaf_install.log"
    local result_code = 40000

    local resp_json= {
        code=20000,
        data={}
    }
   

    http.prepare_content("application/json") 
    local ipk_url= luci.http.formvalue("ipk_url")          
    f_log("install ipk url: "..ipk_url)
    local result = luci.sys.exec("oaf_mgr upgrade "..ipk_url )

    if fs.access(result_file) then
        result_code = fs.readfile(result_file)
        resp_json.code = tonumber(result_code)
        f_log("install ipk result: "..result_code)
    else
        f_log("install ipk result file not exist")
        resp_json.code = 40000
    end

    if fs.access(result_log_file) then
        result_log = fs.readfile(result_log_file)
        resp_json.data.error_log = result_log
    else
        f_log("install ipk result log file not exist")
        resp_json.data.error_log = ""
    end
    
   http.write_json(resp_json)
end
